﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web;
using System.Web.Http;
using GovDataAPI.Models;
using System.Data.Spatial;

namespace GovDataAPI.Controllers
{
    public class ItemsController : ApiController
    {
        private GovDataStoreEntities db = new GovDataStoreEntities();

        // GET api/Items
        public IEnumerable<Item> GetItems()
        {
            var items = db.Items.Include(i => i.Source);
            return items.AsEnumerable();
        }

        // GET api/Items/5
        public Item GetItem(int id)
        {
            Item item = db.Items.Find(id);
            if (item == null)
            {
                throw new HttpResponseException(Request.CreateResponse(HttpStatusCode.NotFound));
            }

            return item;
        }

        /// <summary>
        /// Returns content from the items table within the specified radius
        /// </summary>
        /// <param name="latitude">Reference Latitude</param>
        /// <param name="longitude">Reference Longitude</param>
        /// <param name="radius">Search Radius in miles</param>
        /// <param name="text">Optional: Item text</param>
        /// <returns></returns>
        public IEnumerable<SearchResult> GetItems(double latitude, double longitude, double radius, string text = "")
        {
            DbGeography point = DbGeography.PointFromText(String.Format("POINT({0} {1})", longitude, latitude)
                , DbGeography.DefaultCoordinateSystemId);
            double distance = MilesToMeters(radius);
            List<SearchResult> results = new List<SearchResult>();

            db.Items.Where(
                i => (i.Title.Contains(text) || i.Description.Contains(text) || String.IsNullOrEmpty(text))
                    && i.Location.Distance(point) < distance)
                .ToList()
                .ForEach(r =>
                {
                    results.Add(new SearchResult(r));
                }
             );

            return results.AsEnumerable();
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        /// ref: Basic Spatial Data with SQL Server and Entity Framework 5.0 - by Rick Strahl  - http://bit.ly/MuaLUS 
        /// 
        /// Convert meters to miles
        /// 
        /// 
        /// 
        public static double MetersToMiles(double? meters)
        {
            if (meters == null)
                return 0F;

            return meters.Value * 0.000621371192;
        }

        /// 
        /// Convert miles to meters
        /// 
        /// 
        /// 
        public static double MilesToMeters(double? miles)
        {
            if (miles == null)
                return 0;

            return miles.Value * 1609.344;
        }
    }
}