﻿using GovDataVisApp.Commands;
using GovDataVisApp.Models;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using System.Xml.Linq;

namespace GovDataVisApp.ViewModels
{
    public class MapSearchViewModel: INotifyPropertyChanged
    {
        

        public MapSearchViewModel()
        {
            this.SearchResults = new ObservableCollection<MapSearchResult>();
            this.IsWorking = false;

            // set up the search results load command
            this._loadSearchResults = new DelegateCommand(this.LoadGeoSearchApiResults);

            this._query = new MapQuery();
        }

        private string _baseAddressUrl;
        public string BaseAddressUrl
        {
            get
            {
                return this._baseAddressUrl;
            }

            set
            {
                this._baseAddressUrl = value;
                this.NotifyPropertyChanged();
            }
        }

        private bool _isWorking = false;
        public bool IsWorking
        {
            get { return this._isWorking; }
            set
            {
                this._isWorking = value;
                this.NotifyPropertyChanged();
            }
        }

        private bool _queryComplete = false;
        public bool QueryComplete
        {
            get { return this._queryComplete; }
            set
            {
                this._queryComplete = value;
                this.NotifyPropertyChanged();
            }
        }

        private ICommand _loadSearchResults;
        public ICommand LoadSearchResults
        {
            get
            {
                return this._loadSearchResults;
            }
        }

        private MapQuery _query;
        public MapQuery Query
        {
            get { return this._query; }
            set
            {
                this._query = value;
                this.NotifyPropertyChanged();
            }
        }

        // pulled from the msdn wp8 twitter sample: http://code.msdn.microsoft.com/wpapps/TwitterSearch-Windows-b7fc4e5e
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] string propertyName = "")
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private MapSearchResult _selectedResult = null;
        public MapSearchResult SelectedResult
        {
            get { return this._selectedResult; }
            set
            {
                this._selectedResult = value;
                this.NotifyPropertyChanged();
            }
        }

        private async void LoadGeoSearchApiResults(object p)
        {
            try
            {
                // load the results and populate the collection
                this.SearchResults.Clear();

                this.IsWorking = true;
                this.QueryComplete = false;

                HttpClient client = new HttpClient();
                client.BaseAddress = new Uri(this.BaseAddressUrl); // default: http://govdata.azurewebsites.net/

                client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("text/xml"));

                HttpResponseMessage response = await client.GetAsync(String.Format("/api/Items?text={0}&latitude={1}&longitude={2}&radius={3}",
                    new object[] { this.Query.Text, this.Query.Latitude, this.Query.Longitude, this.Query.Radius }));

                using (Stream stream = await response.Content.ReadAsStreamAsync())
                {
                    XDocument doc = XDocument.Load(stream);
                    XNamespace defaultNs = doc.Root.GetDefaultNamespace();
                    var results = doc.Descendants(defaultNs + "SearchResult");

                    foreach (var result in results)
                    {
                        this.SearchResults.Add(new MapSearchResult()
                        {
                            Title = result.Descendants(defaultNs + "Title").First().Value,
                            Description = result.Descendants(defaultNs + "Description").First().Value,
                            Latitude = Convert.ToDouble(result.Descendants(defaultNs + "Latitude").First().Value),
                            Longitude = Convert.ToDouble(result.Descendants(defaultNs + "Longitude").First().Value)
                        });
                    }
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {                
                this.QueryComplete = true;
                this.IsWorking = false;
            }
        }

        public ObservableCollection<MapSearchResult> SearchResults
        {
            get;
            private set;
        }
    }
}
